% This matlab codes implement the adaptive residual subsampling 
% method for radial basis function 1-D initial-boundary value problem. 
%
% Implementation : Method of lines:
%                  Use ode15s as time stepping.
%                  Space adaptivity: Adaptive residual subsampling for RBFs
%
% For reference, see:
% Adaptive residual subsampling methods for radial basis function
% interpolation and collocation problems. submitted to Computers Math. Appl
%
% Tobin A. Driscoll and Alfa R.H. Heryudono    05/14/2006
% MATLAB 7 is recommended.
%
% Test problem:
% Burger's Equation
% epsilon.u_xx - u.u_x = u_t, 0 < x < 1
% u(0,t)=u(1,t)=0.

clear all;close all;
Tfin = 1;
NT = 51;

epsilon = 1e-3; theta = 1e-5;
cg = 0.5;
theta = [theta theta/1000];
initcond = @(x) sin(2*pi*x) + 0.5*sin(pi*x);
[x,c,u0] = coarserefine(initcond,[0 1],theta,13,cg);

xx = linspace(0,1,301)';
uu = initcond(xx);

han = plot(x,u0,'o',xx,uu,'-r',x,-1*exp(0*x),'ko','MarkerFaceColor','k','MarkerSize',2);
tp = title('');hold on;
xlabel('x');ylabel('u(x,t)');
axis([0 1 -1 1.5])

Tspan = linspace(0,Tfin,NT);
set(tp,'string',sprintf('T = %.3f,    N = %3i.',Tspan(1),length(x)));

for i=1:NT-1    
N = length(x);
[A,D1,D2] = deal(zeros(N));
for j=1:N
    [A(:,j),D1(:,j),D2(:,j)] = mq(x,x(j),c(j));
end

lambda = A\eye(N);
D1 = D1*lambda; D1([1 N],:)=[]; D1(:,[1 N])=[];
D2 = D2*lambda; D2([1 N],:)=[]; D2(:,[1 N])=[];

% Solve system of ODEs
options = odeset('RelTol',1e-6,'AbsTol',1e-8,'Jacobian',@Jburgers,'OutputFcn', ...
                  @(t,y,flag,varargin) burgersplot(t,y,flag,varargin,x,N,xx,han,tp));

set(tp,'string',sprintf('T = %.3f,    N = %3i.',Tspan(i+1),N));
[T,U] = ode15s(@burgers,[Tspan(i) Tspan(i+1)],u0(2:N-1),options,epsilon,D1,D2);

disp('-----------------------');
[x,c,u0] = coarserefine(@(xp)predictor(xp,x,[0;U(end,:)';0],lambda,c),[0 1],theta,13,cg);
end

